@@ -28,56 +28,77 @@
  *      Riley C. Porter <Riley.Porter@g2-inc.com>
  */
 
-#include "cce.h"
+#include "public/cce.h"
 #include "cce_priv.h"
+#include "oscap.h"
 
-void cce_init(struct cce *cce)
+#define CCE_SUPPORTED "5"
+
+static struct cce *cce_new_empty(void)
 {
-	cce->id = NULL;
-	cce->description = NULL;
-	cce->parameters =
-	    (struct list_cstring *)malloc(sizeof(struct list_cstring));
-	cce->technicalmechanisms =
-	    (struct list_cstring *)malloc(sizeof(struct list_cstring));
-	cce->parameters->next = NULL;
-	cce->parameters->value = NULL;
-	cce->technicalmechanisms->next = NULL;
-	cce->technicalmechanisms->value = NULL;
-	cce->references = (struct list_refs *)malloc(sizeof(struct list_refs));
-	cce->references->next = NULL;
-	cce->references->value = NULL;
-	cce->references->source = NULL;
-	return;
+	struct cce *cce = oscap_calloc(1, sizeof(struct cce));
+	cce->entries = oscap_list_new();
+	cce->entry = oscap_htable_new();
+	return cce;
 }
 
-void cce_clear(struct cce *cce)
+void cce_free(struct cce *cce)
 {
-	if (cce->id)
-		xmlFree(cce->id);
-	if (cce->description)
-		xmlFree(cce->description);
-	list_cstring_clear(cce->parameters);
-	list_cstring_clear(cce->technicalmechanisms);
-	list_refs_clear(cce->references);
+	if (cce) {
+		oscap_htable_free(cce->entry, NULL);
+		oscap_list_free(cce->entries, (oscap_destruct_func) cce_entry_free);
+		oscap_free(cce);
+	}
 }
 
-void cce_parse(char *docname, struct cce *cce, char *id)
+void cce_entry_free(struct cce_entry *cce)
+{
+	if (cce) {
+		oscap_free(cce->id);
+		oscap_free(cce->description);
+		oscap_list_free(cce->params, oscap_free);
+		oscap_list_free(cce->tech_mechs, oscap_free);
+		oscap_list_free(cce->references, (oscap_destruct_func) cce_reference_free);
+		oscap_free(cce);
+	}
+}
+
+struct cce_entry *cce_entry_new_empty(void)
+{
+	struct cce_entry *cce = oscap_calloc(1, sizeof(struct cce_entry));
+	cce->params = oscap_list_new();
+	cce->tech_mechs = oscap_list_new();
+	cce->references = oscap_list_new();
+	return cce;
+}
+
+static void cce_parse(const char *docname, struct cce *cce)
 {
 	xmlTextReaderPtr reader;
-	xmlDocPtr doc;
 	int ret;
-	doc = xmlParseFile(docname);
 	reader = xmlNewTextReaderFilename(docname);
 	if (reader != NULL) {
 		ret = xmlTextReaderRead(reader);
 		while (ret == 1) {
-			process_node(reader, cce, id);
+			process_node(reader, cce);
 			ret = xmlTextReaderRead(reader);
-
 		}
+		xmlFreeTextReader(reader);
 	}
 }
 
+struct cce *cce_new(const char *fname)
+{
+	struct cce *cce = cce_new_empty();
+	cce_parse(fname, cce);
+	return cce;
+}
+
+const char * cce_supported(void)
+{
+        return CCE_SUPPORTED;
+}
+
 bool cce_validate(const char *filename)
 {
 	xmlParserCtxtPtr ctxt;	/* the parser context */
@@ -103,3 +124,13 @@ bool cce_validate(const char *filename)
 	return ret;
 
 }
+
+OSCAP_IGETTER_GEN(cce_entry, cce, entries)
+    OSCAP_HGETTER(struct cce_entry *, cce, entry)
+
+
+OSCAP_GETTER(const char *, cce_entry, id)
+OSCAP_GETTER(const char *, cce_entry, description)
+OSCAP_IGETTER(oscap_string, cce_entry, params)
+OSCAP_IGETTER(oscap_string, cce_entry, tech_mechs) OSCAP_IGETTER_GEN(cce_reference, cce_entry, references)
+ OSCAP_GETTER(const char *, cce_reference, source) OSCAP_GETTER(const char *, cce_reference, value)
